from serpapi.serp_api_client import *
from serpapi.serp_api_client_exception import SerpApiClientException

class GoogleScholarSearch(SerpApiClient):
    """GoogleScholarSearch enables to search google scholar and parse the result.
    ```python
    from serpapi import GoogleScholarSearch
    query = GoogleScholarSearch({"q": "coffee"})
    data = query.get_json()
    ```

    doc: https://serpapi.com/google-scholar-api
    """

    def __init__(self, params_dict):
        super(GoogleScholarSearch, self).__init__(params_dict, GOOGLE_SCHOLAR_ENGINE)

    def get_location(self, q, limit = 5):
        raise SerpApiClientException("location is not supported by Google scholar search engine")



from serpapi.serp_api_client import *
from serpapi.serp_api_client_exception import SerpApiClientException

class HomeDepotSearch(SerpApiClient):
    """HomeDepotSearch enables to search home depot and parse the result.
    ```python
    from serpapi import HomeDepotSearch
    query = HomeDepotSearch({"q": "chair"})
    data = query.get_json()
    ```

    doc: https://serpapi.com/home-depot-search-api
    """

    def __init__(self, params_dict):
        super(HomeDepotSearch, self).__init__(params_dict, HOME_DEPOT_ENGINE)

    def get_location(self, q, limit = 5):
        raise SerpApiClientException("location is not supported by Home Depot search engine")



from .serp_api_client import SerpApiClient
from .baidu_search import BaiduSearch
from .google_search import GoogleSearch
from .yahoo_search import YahooSearch
from .bing_search import BingSearch
from .yandex_search import YandexSearch
from .google_scholar_search import GoogleScholarSearch
from .ebay_search import EbaySearch
from .home_depot_search import HomeDepotSearch
from .youtube_search import YoutubeSearch
from .duck_duck_go_search import DuckDuckGoSearch
from .walmart_search import WalmartSearch
from .naver_search import NaverSearch
from .apple_app_store_search import AppleAppStoreSearch




import random
import unittest
import os
from serpapi import GoogleSearch

class TestAccountApi(unittest.TestCase):

    def setUp(self):
        GoogleSearch.SERP_API_KEY = os.getenv("API_KEY","demo")

    @unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
    def test_get_account(self):
        search = GoogleSearch({})
        account = search.get_account()
        self.assertIsNotNone(account.get("account_id"))
        self.assertEqual(account.get("api_key"), GoogleSearch.SERP_API_KEY)

if __name__ == '__main__':
    unittest.main()



import random
import unittest
import os
import pprint
from serpapi import YoutubeSearch

class TestYoutubeSearchApi(unittest.TestCase):

		def setUp(self):
				YoutubeSearch.SERP_API_KEY = os.getenv("API_KEY", "demo")

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_json(self):
				search = YoutubeSearch({"search_query": "chair"})
				data = search.get_dict()
				self.assertIsNone(data.get("error"))
				self.assertEqual(data["search_metadata"]["status"], "Success")
				self.assertIsNotNone(data["search_metadata"]["youtube_url"])
				self.assertIsNotNone(data["search_metadata"]["id"])

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_object(self):
				search = YoutubeSearch({"search_query": "chair"})
				data = search.get_object()
				self.assertEqual(data.search_metadata.status, "Success")
				self.assertIsNotNone(data.search_metadata.id)
				self.assertIsNotNone(data.search_metadata.youtube_url)
				self.assertEqual(data.search_parameters.search_query, "chair")
				self.assertEqual(data.search_parameters.engine, "youtube")
				self.assertGreater(data.search_information.total_results, 10)

if __name__ == '__main__':
		unittest.main()



from serpapi.serp_api_client import *
from serpapi.serp_api_client_exception import SerpApiClientException

class YandexSearch(SerpApiClient):
    """YandexSearch enables to search yandex and parse the result.
    ```python
    from serpapi import YandexSearch
    query = YandexSearch({"text": "coffee"})
    data = query.get_json()
    ```

    doc: https://serpapi.com/yandex-search-api
    """

    def __init__(self, params_dict):
        super(YandexSearch, self).__init__(params_dict, YANDEX_ENGINE)

    def get_location(self, q, limit = 5):
        raise SerpApiClientException("location is not supported by Yandex search engine at this time")



import random
import unittest
import os
import pprint
from serpapi import AppleAppStoreSearch

class TestAppleAppStoreSearch(unittest.TestCase):

		def setUp(self):
				AppleAppStoreSearch.SERP_API_KEY = os.getenv("API_KEY", "demo")

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_json(self):
				search = AppleAppStoreSearch({"term": "Coffee"})
				data = search.get_dict()
				self.assertEqual(data["search_metadata"]["status"], "Success")
				#self.assertIsNotNone(data["search_metadata"]["app_store_url"])
				self.assertIsNotNone(data["search_metadata"]["id"])
				if "organic_results" in data:
					self.assertIsNotNone(data["organic_results"][1]["title"])
				pp = pprint.PrettyPrinter(indent=2)
				pp.pprint(data)
				print(data.keys())

if __name__ == '__main__':
		unittest.main()



from serpapi.serp_api_client import *
from serpapi.serp_api_client_exception import SerpApiClientException
from serpapi.constant import *

class NaverSearch(SerpApiClient):
    """NaverSearch enables to search google scholar and parse the result.
    ```python
    from serpapi import NaverSearch
    search = NaverSearch({"query": "chair"})
    data = search.get_dict()
    ```

    doc: https://serpapi.com/naver-search-api
    """

    def __init__(self, params_dict):
        super(NaverSearch, self).__init__(params_dict, NAVER_ENGINE)

    def get_location(self, q, limit = 5):
        raise SerpApiClientException("location is not supported by youtube search engine")



import requests
import json
from serpapi.pagination import Pagination
from serpapi.serp_api_client_exception import SerpApiClientException

GOOGLE_ENGINE = 'google'
BING_ENGINE = 'bing'
BAIDU_ENGINE = 'baidu'
GOOGLE_SCHOLAR_ENGINE = 'google_scholar'
YANDEX_ENGINE = 'yandex'
EBAY_ENGINE = 'ebay'
YAHOO_ENGINE = 'yahoo'
HOME_DEPOT_ENGINE = 'home_depot'
YOUTUBE_ENGINE = 'youtube'

class SerpApiClient(object):
    """SerpApiClient enables to query any search engines supported by SerpApi and parse the results.
    ```python
    from serpapi import GoogleSearch
    search = SerpApiClient({
        "q": "Coffee", 
        "location": "Austin,Texas", 
        "engine": "google",
        "api_key": "<your private key>"
        })
	data = search.get_json()
    ```

    https://serpapi.com/search-api
    """

    BACKEND = "https://serpapi.com"
    SERP_API_KEY = None

    def __init__(self, params_dict, engine = None, timeout = 60000):
        self.params_dict = params_dict
        self.engine = engine
        self.timeout = timeout

    def construct_url(self, path = "/search"):
        self.params_dict['source'] = 'python'
        if self.SERP_API_KEY:
            self.params_dict['serp_api_key'] = self.SERP_API_KEY
        if self.engine:
            if not 'engine' in self.params_dict:
                self.params_dict['engine'] = self.engine
        if not 'engine' in self.params_dict:
            raise SerpApiClientException("engine must be defined in params_dict or engine")
        return self.BACKEND + path, self.params_dict

    def get_response(self, path = '/search'):
        """Returns:
            Response object provided by requests.get
        """
        url = None
        try:
            url, parameter = self.construct_url(path)
            # print(url)
            response = requests.get(url, parameter, timeout=self.timeout)
            return response
        except requests.HTTPError as e:
            print("fail: " + url)
            print(e, e.response.status_code)
            raise e

    def get_results(self, path='/search'):
        """Returns:
            Response text field
        """
        return self.get_response(path).text

    def get_html(self):
        """Returns:
            Raw HTML search result from Gooogle
        """
        return self.get_results()

    def get_json(self):
        """Returns:
            Formatted JSON search results using json package
        """
        self.params_dict["output"] = "json"
        return json.loads(self.get_results())

    def get_raw_json(self):
        """Returns:
            Formatted JSON search result as string
        """
        self.params_dict["output"] = "json"
        return self.get_results()

    def get_dictionary(self):
        """Returns:
            Dict with the formatted response content
        """
        return dict(self.get_json())

    def get_dict(self):
        """Returns:
            Dict with the formatted response content
            (alias for get_dictionary)
        """
        return self.get_dictionary()

    def get_object(self):
        """Returns: 
            Dynamically created python object wrapping the result data structure
        """
        # iterative over response hash
        node = self.get_dictionary()
        # create dynamic python object
        return self.make_pyobj("response", node)

    def make_pyobj(self, name, node):
        pytype = type(name, (object, ), {})
        pyobj = pytype()

        if isinstance(node, list):
            setattr(pyobj, name, [])
            for el in node:
                getattr(pyobj, name).append(self.make_pyobj(name, el))
            return pyobj
        elif isinstance(node, dict):
            for name, child in node.items():
                if isinstance(child, list):
                    setattr(pyobj, name, [])
                    for el in child:
                        getattr(pyobj, name).append(self.make_pyobj(name, el))
                elif isinstance(child, dict):
                    setattr(pyobj, name, self.make_pyobj(name, child))
                else:
                    setattr(pyobj, name, child)
        else:
            setattr(pyobj, name, node)

        return pyobj

    def get_search_archive(self, search_id, format = 'json'):
        """Retrieve search result from the Search Archive API
        Parameters:
            search_id (int): unique identifier for the search provided by metadata.id 
            format (string): search format: json or html [optional]
        Returns:
            dict|string: search result from the archive
        """
        result = self.get_results("/searches/{0}.{1}".format(search_id, format))
        if format == 'json':
            result = json.loads(result)
        return result

    def get_account(self):
        """Get account information using Account API
        Returns:
            dict: account information
        """
        return json.loads(self.get_results("/account"))

    def get_location(self, q, limit = 5):
        """Get location using Location API
        Parameters:
            q (string): location (like: city name..)
            limit (int): number of matches returned
        Returns:
            dict: Location matching q
        """
        self.params_dict = {}
        self.params_dict["output"] = "json"
        self.params_dict["q"] = q
        self.params_dict["limit"] = limit
        buffer = self.get_results('/locations.json')
        return json.loads(buffer)
    
    def pagination(self, start = 0, end = 1000000000, page_size = 10):
        """Return:
            Generator to iterate the search results pagination
        """
        return Pagination(self, start, end, page_size)




# Pagination constant
DEFAULT_START = 0
DEFAULT_END = 100
DEFAULT_PAGE_SIZE = 10

# Supported earch engine
GOOGLE_ENGINE = 'google'
GOOGLE_SCHOLAR_ENGINE = 'google_scholar'
BAIDU_ENGINE = 'baidu'
BING_ENGINE = 'bing'
YANDEX_ENGINE = 'yandex'
EBAY_ENGINE = 'ebay'
YAHOO_ENGINE = 'yahoo'
HOME_DEPOT_ENGINE = 'home_depot'
YOUTUBE_ENGINE = 'youtube'
DUCKDUCKGO_ENGINE = 'duckduckgo'
WALMART_ENGINE = "walmart"
NAVER_ENGINE = "naver"
APPLE_APP_STORE_ENGINE = "apple_app_store"

# from serpapi.constant import *


# Unit testing
import unittest

# Operating system
import os

# regular expression library
import re

# safe queue 
import sys
if (sys.version_info > (3, 0)):
  from queue import Queue
else:
  from Queue import Queue
  
# Time utility
import time

# Serp API search
from serpapi import GoogleSearch

# download file with wget
#import wget

class TestExample(unittest.TestCase):

    def setUp(self):
        GoogleSearch.SERP_API_KEY = os.getenv("API_KEY","demo")

    @unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
    def test_get_json(self):
        search = GoogleSearch({"q": "Coffee", "engine": "google_scholar"})
        data = search.get_json()
        print(data['search_metadata'])
        search_id = data['search_metadata']['id']
        # retrieve search from the archive - blocker
        print(search_id + ": get search from archive")
        raw_html =  search.get_search_archive(search_id, 'html')
        # print(search_id + ": status = " + search_archived['search_metadata']['status'])
        print(raw_html)
        #print(search_html)

    @unittest.skipIf((os.getenv("DEBUGAPI_KEY") == None), "no api_key provided")
    def test_search_google_images(self):
        search = GoogleSearch({"q": "coffe", "tbm": "isch"})
        for image_result in search.get_json()['images_results']:
            try:
                link = image_result["original"]
                print("link is found: " + link)
                # uncomment the line below to down the original image
                # wget.download(link, '.')
            except:
                print("link is not found.")
                pass
            # https://github.com/serpapi/showcase-serpapi-tensorflow-keras-image-training/blob/master/fetch.py

    @unittest.skipIf((os.getenv("DEBUGAPI_KEY") == None), "no api_key provided")
    def test_async(self):
        # store searches
        search_queue = Queue()
        
        # Serp API search
        search = GoogleSearch({
            "location": "Austin,Texas",
            "async": True
        })
        
        # loop through companies
        for company in ['amd','nvidia','intel']:
          print("execute async search: q = " + company)
          search.params_dict["q"] = company
          data = search.get_dict()
          if data is not None:
              print("oops data is empty for: " + company)
              continue
          print("add search to the queue where id: " + data['search_metadata']['id'])
          # add search to the search_queue
          search_queue.put(data)
        
        print("wait until all search statuses are cached or success")
        
        # Create regular search
        search = GoogleSearch({"async": True})
        while not search_queue.empty():
          data = search_queue.get()
          search_id = data['search_metadata']['id']

          # retrieve search from the archive - blocker
          print(search_id + ": get search from archive")
          search_archived =  search.get_search_archive(search_id)
          print(search_id + ": status = " + search_archived['search_metadata']['status'])
          
          # check status
          if re.search('Cached|Success', search_archived['search_metadata']['status']):
            print(search_id + ": search done with q = " + search_archived['search_parameters']['q'])
          else:
            # requeue search_queue
            print(search_id + ": requeue search")
            search_queue.put(search)
            # wait 1s
            time.sleep(1)
        # search is over.
        print('all searches completed')
        
    @unittest.skipIf((os.getenv("DEBUGAPI_KEY") == None), "no api_key provided")
    def test_search_google_news(self):
        search = GoogleSearch({
            "q": "coffe",   # search search
            "tbm": "nws",   # news
            "tbs": "qdr:d", # last 24h
            "num": 10
        })
        for offset in [0,1,2]:
            search.params_dict["start"] = offset * 10
            data = search.get_json()
            for news_result in data['news_results']:
                print(str(news_result['position'] + offset * 10) + " - " + news_result['title'])

    @unittest.skipIf((os.getenv("DEBUGAPI_KEY") == None), "no api_key provided")
    def test_search_google_shopping(self):
        search = GoogleSearch({
            "q": "coffe",   # search search
            "tbm": "shop",  # news
            "tbs": "p_ord:rv", # last 24h
            "num": 100
        })
        data = search.get_json()
        if 'shopping_results' in data:
            for shopping_result in data['shopping_results']:
                print(str(shopping_result['position']) + " - " + shopping_result['title'])
        else:
            print("WARNING: oops shopping_results is missing from search result with tbm=shop")
    
    @unittest.skipIf((os.getenv("DEBUGAPI_KEY") == None), "no api_key provided")
    def test_search_by_location(self):
        for city in ["new york", "paris", "berlin"]:
            location = GoogleSearch({}).get_location(city, 1)[0]["canonical_name"]
            search = GoogleSearch({
                "q": "best coffee shop",   # search search
                "location": location,
                "num": 10,
                "start": 0
            })
            data = search.get_json()
            self.assertIsNone(data.get("error"))
            top_result = data['organic_results'][0]["title"]
            print("top coffee result for " + location + " is: " + top_result)


if __name__ == '__main__':
    unittest.main()



from serpapi.serp_api_client import *

class GoogleSearch(SerpApiClient):
    """GoogleSearch enables to search google and parse the result.
    ```python
    from serpapi import GoogleSearch
    query = GoogleSearch({"q": "coffee", "location": "Austin,Texas"})
    data = query.get_json()
    ```

    https://github.com/serpapi/google-search-results-python
    """

    def __init__(self, params_dict):
        super(GoogleSearch, self).__init__(params_dict, GOOGLE_ENGINE)



import random
import unittest
import os
import pprint
import requests
from serpapi import GoogleSearch

class TestSearchApi(unittest.TestCase):

		def setUp(self):
				GoogleSearch.SERP_API_KEY = os.getenv("API_KEY", "demo")

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_paginate(self):
				search = GoogleSearch({"q": "Coffee", "location": "Austin,Texas"})
				pages = search.pagination(0, 20, 10)
				urls = []
				for page in pages:
					urls.append(page['serpapi_pagination']['next'])
				self.assertEqual(len(urls), 2)
				self.assertTrue("start=10" in urls[0])
				print(urls[1])
				self.assertTrue("start=21" in urls[1])

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_json(self):
				search = GoogleSearch({"q": "Coffee", "location": "Austin,Texas"})
				data = search.get_json()
				self.assertEqual(data["search_metadata"]["status"], "Success")
				self.assertIsNone(data.get("error"))
				self.assertIsNotNone(data["search_metadata"]["google_url"])
				self.assertIsNotNone(data["search_metadata"]["id"])
				self.assertIsNotNone(data['local_results']['places'][0])

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_json(self):
				search = GoogleSearch({"q": "Coffee", "engine": "google_scholar"})
				data = search.get_json()
				self.assertIsNotNone(data["organic_results"][0]["title"])

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_dict(self):
				search = GoogleSearch({"q": "Coffee", "location": "Austin,Texas"})
				data = search.get_dict()
				self.assertIsNotNone(data.get('local_results'))

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_dictionary(self):
				search = GoogleSearch({"q": "Coffee", "location": "Austin,Texas"})
				data = search.get_dictionary()
				self.assertIsNotNone(data.get('local_results'))

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_html(self):
				search = GoogleSearch({"q": "Coffee", "location": "Austin,Texas"})
				data = search.get_html()
				self.assertGreater(len(data), 10)

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_response(self):
				search = GoogleSearch({"q": "Coffee", "location": "Austin,Texas"})
				response = search.get_response()
				self.assertEqual(type(response), requests.Response)
				self.assertGreater(len(response.text), 10)

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_object(self):
				search = GoogleSearch({"q": "Coffee", "location": "Austin,Texas"})
				r = search.get_object()
				self.assertEqual(type(r.organic_results), list)
				self.assertIsNotNone(r.organic_results[0].title)
				self.assertIsNotNone(r.search_metadata.id)
				self.assertIsNotNone(r.search_metadata.google_url)
				self.assertEqual(r.search_parameters.q, "Coffee")
				self.assertEqual(r.search_parameters.engine, "google")
				self.assertGreater(r.search_information.total_results, 10)

if __name__ == '__main__':
		unittest.main()



import random
import unittest
import os
import pprint
from serpapi import DuckDuckGoSearch

class TestDuckDuckGoSearch(unittest.TestCase):

		def setUp(self):
				DuckDuckGoSearch.SERP_API_KEY = os.getenv("API_KEY", "demo")

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_json(self):
				search = DuckDuckGoSearch({"q": "Coffee"})
				data = search.get_json()
				self.assertIsNone(data.get("error"))
				self.assertEqual(data["search_metadata"]["status"], "Success")
				self.assertIsNotNone(data["search_metadata"]["duckduckgo_url"])
				self.assertIsNotNone(data["search_metadata"]["id"])
				if "organic_results" in data:
					self.assertIsNotNone(data["organic_results"][1]["title"])
				# pp = pprint.PrettyPrinter(indent=2)
				# pp.pprint(data)
				self.assertTrue(len(data.keys()) > 3)

if __name__ == '__main__':
		unittest.main()



from serpapi.serp_api_client import *
from serpapi.serp_api_client_exception import SerpApiClientException
from serpapi.constant import *

class AppleAppStoreSearch(SerpApiClient):
    """AppleAppStoreSearch enables to search google scholar and parse the result.
    ```python
    from serpapi import AppleAppStoreSearch
    search = AppleAppStoreSearch({"term": "ipad"})
    data = search.get_dct()
    ```
    
    doc: https://serpapi.com/apple-app-store
    """

    def __init__(self, params_dict):
        super(AppleAppStoreSearch, self).__init__(params_dict, APPLE_APP_STORE_ENGINE)

    def get_location(self, q, limit = 5):
        raise SerpApiClientException("location is not supported by youtube search engine")



from serpapi.serp_api_client import *
from serpapi.serp_api_client_exception import SerpApiClientException

class BaiduSearch(SerpApiClient):
    """BaiduSearch enables to search baidu and parse the result.
    ```python
    from serpapi import BaiduSearch
    query = BaiduSearch({"q": "coffee"})
    data = query.get_json()
    ```

    doc: https://serpapi.com/baidu-search-api
    """

    def __init__(self, params_dict):
        super(BaiduSearch, self).__init__(params_dict, BAIDU_ENGINE)

    def get_location(self, q, limit = 5):
        raise SerpApiClientException("location is not supported by Baidu search engine at this time")



from serpapi.serp_api_client import *
from serpapi.serp_api_client_exception import SerpApiClientException

class YoutubeSearch(SerpApiClient):
    """YoutubeSearch enables to search google scholar and parse the result.
    ```python
    from serpapi import YoutubeSearch
    query = YoutubeSearch({"search_query": "chair"})
    data = query.get_dict()
    ```

    doc: https://serpapi.com/youtube-search-api
    """

    def __init__(self, params_dict):
        super(YoutubeSearch, self).__init__(params_dict, YOUTUBE_ENGINE)

    def get_location(self, q, limit = 5):
        raise SerpApiClientException("location is not supported by youtube search engine")



import random
import unittest
import os
import pprint
from serpapi import SerpApiClient
from serpapi.serp_api_client_exception import SerpApiClientException
import pytest

# This test shows how to extends SerpApiClient
#  without using search engine wrapper.
# 
class TestSerpSearchApi(unittest.TestCase):

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_json(self):
			search = SerpApiClient({
				"q": "Coffee", 
				"location": "Austin,Texas", 
				"engine": "google_scholar",
				"api_key": os.getenv("API_KEY")
				})
			data = search.get_json()
			assert data.get("error") == None
			self.assertIsNotNone(data["organic_results"][0]["title"])

		def test_invalid_api_key(self):
			search = SerpApiClient({
				"q": "Coffee", 
				"location": "USA", 
				"engine": "google",
				"api_key": "invalid_api_key"
			})
			data = search.get_json()
			self.assertIsNotNone(data["error"])

		def test_error_missing_engine(self):
			search = SerpApiClient({
				"q": "Coffee", 
				"api_key": os.getenv("API_KEY")
			})
			with pytest.raises(SerpApiClientException):
				search.get_json()

		def test_missing_(self):
			search = SerpApiClient({
				"location": "USA", 
				"engine": "google",
				"api_key": os.getenv("API_KEY")
			})
			data = search.get_json()
			self.assertIsNotNone(data["error"])
			self.assertRegex(data["error"], "Missing query `q`")

		def debug(self, payload):
			pp = pprint.PrettyPrinter(indent=2)
			pp.pprint(payload)

if __name__ == '__main__':
		unittest.main()



import random
import unittest
import os
import pprint
from serpapi import EbaySearch

class TestEbaySearchApi(unittest.TestCase):

		def setUp(self):
				EbaySearch.SERP_API_KEY = os.getenv("API_KEY", "demo")

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_json(self):
				search = EbaySearch({"_nkw": "Coffee"})
				data = search.get_json()
				self.assertIsNone(data.get("error"))
				self.assertEqual(data["search_metadata"]["status"], "Success")
				self.assertIsNotNone(data["search_metadata"]["ebay_url"])
				self.assertIsNotNone(data["search_metadata"]["id"])
				self.assertIsNotNone(data["organic_results"][0]["title"])
				# pp = pprint.PrettyPrinter(indent=2)
				# pp.pprint(data)

    # TODO fix universal pagination
		@unittest.skipIf((os.getenv("DEBUGAPI_KEY") == None), "no api_key provided")
		def test_paginate(self):
			params = {
				"_nkw": "coffee",
				"api_key": os.getenv("API_KEY")
			}
			search = EbaySearch(params)
			pages = search.pagination(20, 60)
			page_count = 0
			result_count = 0
			for page in pages:
				page_count += 1
				result_count += len(page["organic_results"])
			self.assertEqual(page_count, 2)
			self.assertEqual(page_count, 40)

if __name__ == '__main__':
		unittest.main()



import random
import unittest
import os
import pprint
from serpapi import YandexSearch

class TestYandexSearchApi(unittest.TestCase):

		def setUp(self):
				YandexSearch.SERP_API_KEY = os.getenv("API_KEY", "demo")

		@unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
		def test_get_json(self):
				search = YandexSearch({"text": "Coffee"})
				data = search.get_json()
				self.assertIsNone(data.get("error"))
				self.assertEqual(data["search_metadata"]["status"], "Success")
				self.assertIsNotNone(data["search_metadata"]["yandex_url"])
				self.assertIsNotNone(data["search_metadata"]["id"])
				pp = pprint.PrettyPrinter(indent=2)
				pp.pprint(data["organic_results"])
				self.assertIsNotNone(data["organic_results"][1]["title"])

if __name__ == '__main__':
		unittest.main()



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open('README.md', mode='r', encoding='utf-8') as fp:
    README = fp.read()

setup(name='google_search_results',
      version='2.4.2',
      description='Scrape and search localized results from Google, Bing, Baidu, Yahoo, Yandex, Ebay, Homedepot, youtube at scale using SerpApi.com',
      url='https://github.com/serpapi/google-search-results-python',
      author='vikoky',
      author_email='victor@serpapi.com',
      classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Natural Language :: English',
        'Topic :: Utilities',
        ],
    python_requires='>=3.5',
    zip_safe=False,
    include_package_data=True,
    license="MIT",
    install_requires = ["requests"],
    packages=find_packages(),
    keywords='scrape,serp,api,json,search,localized,rank,google,bing,baidu,yandex,yahoo,ebay,scale,datamining,training,machine,ml,youtube,naver,walmart,apple,store,app',
    long_description=README,
    long_description_content_type="text/markdown",
)



import random
import unittest
import os
from serpapi import GoogleSearch

class TestSearchArchiveApi(unittest.TestCase):

    def setUp(self):
        GoogleSearch.SERP_API_KEY = os.getenv("API_KEY","demo")

    @unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
    def test_get_search_archive(self):
        search = GoogleSearch({"q": "Coffee", "location": "Austin,Texas"})
        search_result = search.get_dictionary()
        search_id = search_result.get("search_metadata").get("id")
        archived_search_result = GoogleSearch({}).get_search_archive(search_id, 'json')
        self.assertEqual(archived_search_result.get("search_metadata").get("id"), search_id)
        html_buffer = GoogleSearch({}).get_search_archive(search_id, 'html')
        self.assertGreater(len(html_buffer), 10000)

if __name__ == '__main__':
    unittest.main()



import unittest
import os
from serpapi import GoogleSearch

# original code: https://replit.com/@DimitryZub1/Scrape-Google-News-with-Pagination-python-serpapi
class TestExamplePaginate(unittest.TestCase):
    
    @unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
    def test_paginate(self):
      # to get 2 pages
      start = 0
      end = 20
      # basic search parameters
      params = {
        "q": "coca cola",
        "tbm": "nws",
        "api_key": os.getenv("API_KEY")
      }
      # as proof of concept 
      #  title collects
      title = []
      # initialize a search
      search = GoogleSearch(params)
      # create a python generator
      pages = search.pagination(start, end)
      # fetch one search result per iteration 
      #  using a basic python for loop 
      #   which invokes python iterator under the hood.
      page_count = 0
      for page in pages:
        page_count += 1
        #print(f"Current page: {page['serpapi_pagination']['current']}")
        for news_result in page["news_results"]:
            #print(f"Title: {news_result['title']}\nLink: {news_result['link']}\n")
            title.append(news_result['title'])
      # double check if things adds up.
      # total number pages expected
      #  the exact number if variable depending on the search engine backend
      self.assertEqual(page_count, 2)
      self.assertEqual(len(title), 20, "number of search results")
      #self.assertEqual(len(set(title)), len(title), "duplicated elements detected")

    @unittest.skipIf((os.getenv("API_KEY") == None), "no api_key provided")
    def test_paginate_page_size(self):
      # to get 2 pages with each page contains 20 search results
      start = 0
      end = 80
      page_size = 20

      # use parameters in
      params = {
        "q": "coca cola",
        "tbm": "nws",
        "api_key": os.getenv("API_KEY"),
        "start": start,
        "end": end,
        "num": page_size
      }
      title = []
      search = GoogleSearch(params)
      # parameter start,end,page_size will be used instead of pagination
      pages = search.pagination()
      page_count = 0
      count = 0
      for page in pages:
        page_count += 1
        # print(f"Current page: {page['serpapi_pagination']['current']}")
        for news_result in page["news_results"]:
            count += 1
            i = 0
            for t in title:
              i += 1
              if t == news_result['title']:
                print(("%d duplicated title: %s at index: %d" % (count, t, i)))
            #print(f"{count} - title: {news_result['title']}")
            title.append(news_result['title'])

        self.assertEqual(count%2, 0, ("page %s does not contain 20 elements" % page_count))
      
      # check number of pages match
      self.assertEqual(page_count, 4)
      self.assertEqual(len(title), end, "number of search results")
      # google randomly duplicated search result
      # self.assertEqual(len(set(title)), end, "duplicated search results")



from serpapi.serp_api_client import *

class BingSearch(SerpApiClient):
    """BingSearch enables to search bing and parse the result.
    ```python
    from serpapi import BingSearch
    query = BingSearch({"q": "coffee", "location": "Austin,Texas"})
    data = query.get_json()
    ```

    doc: https://serpapi.com/bing-search-api
    """

    def __init__(self, params_dict):
        super(BingSearch, self).__init__(params_dict, BING_ENGINE)



from serpapi.serp_api_client import *
from serpapi.serp_api_client_exception import SerpApiClientException

class EbaySearch(SerpApiClient):
    """EbaySearch enables to search ebay and parse the result.
    ```python
    from serpapi import EbaySearch
    query = EbaySearch({"_nkw": "coffee"})
    data = query.get_json()
    ```

    doc: https://serpapi.com/ebay-search-api
    """

    def __init__(self, params_dict):
        super(EbaySearch, self).__init__(params_dict, EBAY_ENGINE)

    def get_location(self, q, limit = 5):
        raise SerpApiClientException("location is not supported by Ebay search engine at this time")



import random
import unittest
import pprint
import os
from serpapi import GoogleSearch

class TestLocationApi(unittest.TestCase):

    def setUp(self):
        GoogleSearch.SERP_API_KEY = os.getenv("API_KEY","demo")

    def test_get_location(self):
        search = GoogleSearch({"q": None, "async": True})
        location_list = search.get_location("Austin", 3)
        pp = pprint.PrettyPrinter(indent=2)
        pp.pprint(location_list)
        self.assertIsNotNone(location_list[0].get("id"))

if __name__ == '__main__':
    unittest.main()



from serpapi.serp_api_client_exception import SerpApiClientException

DEFAULT_START = 0 
DEFAULT_END = 1000000000
DEFAULT_num = 10

# Paginate response in SearpApi
class Pagination:
  
  def __init__(self, client, start = DEFAULT_START, end = DEFAULT_END, num = DEFAULT_num):
    # serp api client
    self.client = client
    # range
    self.start = start
    self.end = end
    self.num = num

    # use value from the client
    if self.start == DEFAULT_START:
      if 'start' in self.client.params_dict:
        self.start = self.client.params_dict['start']
    if self.end == DEFAULT_END:
      if 'end' in self.client.params_dict:
        self.end = self.client.params_dict['end']
    if self.num == DEFAULT_num:
      if 'num' in self.client.params_dict:
        self.num = self.client.params_dict['num']

    # basic check
    if self.start > self.end:
        raise SerpApiClientException("start: {} must be less than end: {}".format(self.start, self.end))
    if(self.start + self.num) > self.end:
        raise SerpApiClientException("start + num: {} + {} must be less than end: {}".format(self.start, self.num, self.end))

  def __iter__(self):
    self.update()
    return self

  def update(self):
    self.client.params_dict['start'] = self.start
    self.client.params_dict['num'] = self.num
    if self.start > 0:
      self.client.params_dict['start'] += 1

  def __next__(self):
    # update parameter
    self.update()

    # execute request
    result = self.client.get_dict()

    # stop if backend miss to return serpapi_pagination
    if not 'serpapi_pagination' in result:
      raise StopIteration

    # stop if no next page
    if not 'next' in result['serpapi_pagination']:
        raise StopIteration

    # ends if no next page
    if self.start + self.num > self.end:
        raise StopIteration
    
    # increment start page
    self.start += self.num

    return result



from serpapi.serp_api_client import *
from serpapi.serp_api_client_exception import SerpApiClientException
from serpapi.constant import *

class DuckDuckGoSearch(SerpApiClient):
    """DuckDuckGoSearch enables to search google scholar and parse the result.
    ```python
    from serpapi import DuckDuckGoSearch
    search = DuckDuckGoSearch({"query": "chair"})
    data = search.get_json()
    ```

    doc: https://serpapi.com/duckduckgo-search-api
    """

    def __init__(self, params_dict):
        super(DuckDuckGoSearch, self).__init__(params_dict, DUCKDUCKGO_ENGINE)

    def get_location(self, q, limit = 5):
        raise SerpApiClientException("location is not supported by walmart search engine")
